/*
 * MSC (MMC/SD controller) peripheral support.
 *
 * Copyright (C) 2024 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#pragma once

#include <l4/re/c/dma_space.h>
#include <l4/sys/types.h>
#include <stdint.h>



#ifdef __cplusplus

#include <l4/devices/dma-x1600.h>
#include <l4/devices/msc-common.h>

// MMC/SD controller channel.

class Msc_x1600_channel : public Msc_channel
{
protected: 
  Dma_x1600_channel *_dma;
  enum Dma_x1600_request_type _request_type_in, _request_type_out;

public:
  explicit Msc_x1600_channel(l4_addr_t msc_start, l4_addr_t addr,
                             l4_cap_idx_t irq, Dma_x1600_channel *dma,
                             enum Dma_x1600_request_type request_type_in,
                             enum Dma_x1600_request_type request_type_out);

  uint32_t transfer(l4re_dma_space_dma_addr_t from_paddr,
                    l4re_dma_space_dma_addr_t to_paddr,
                    bool recv, uint32_t count);
};

// MMC/SD controller device control.

class Msc_x1600_chip : public Msc_chip
{
protected: 
  unsigned int num_channels()
  { return 2; }

public:
  explicit Msc_x1600_chip(l4_addr_t msc_start, l4_addr_t start, l4_addr_t end);

  Msc_channel *get_channel(uint8_t channel, l4_cap_idx_t irq,
                           Dma_x1600_channel *dma);
};

#endif /* __cplusplus */

/* C language interface. */

EXTERN_C_BEGIN

void *x1600_msc_init(l4_addr_t msc_start, l4_addr_t start, l4_addr_t end);

void *x1600_msc_get_channel(void *msc, uint8_t channel, l4_cap_idx_t irq,
                            void *dma);

uint32_t x1600_msc_get_status(void *msc_channel);

void x1600_msc_enable(void *msc_channel);

uint32_t x1600_msc_read_blocks(void *msc_channel, uint8_t card,
                               l4re_dma_space_dma_addr_t paddr,
                               uint32_t block_address, uint32_t block_count);

EXTERN_C_END
